// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "Explore.h"

#include "ExploreDoc.h"
#include "ExploreTabbedView.h"

#include "ChildFrm.h"
#include "ExploreListView1.h"
#include "ExploreListView2.h"
#include "ExploreListView3.h"
#include "ExploreListView4.h"
#include "ExploreListView5.h"
#include "TestFormView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView

IMPLEMENT_DYNCREATE(CExploreTabbedView, CTabView)

BEGIN_MESSAGE_MAP(CExploreTabbedView, CTabView)
	//{{AFX_MSG_MAP(CExploreTabbedView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_CONTEXTMENU()
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CTabView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CTabView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, OnFilePrintPreview)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_HANDLEVIEWFORM, OnHandleViewForm)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView construction/destruction

CExploreTabbedView::CExploreTabbedView()
	:m_nIndexForm(-1)
{
}

CExploreTabbedView::~CExploreTabbedView()
{
}

BOOL CExploreTabbedView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CTabView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView drawing

void CExploreTabbedView::OnDraw(CDC* /*pDC*/)
{
	CExploreDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView printing

void CExploreTabbedView::OnFilePrintPreview() 
{
	AFXPrintPreview (this);
}

BOOL CExploreTabbedView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CExploreTabbedView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CExploreTabbedView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView diagnostics

#ifdef _DEBUG
void CExploreTabbedView::AssertValid() const
{
	CTabView::AssertValid();
}

void CExploreTabbedView::Dump(CDumpContext& dc) const
{
	CTabView::Dump(dc);
}

CExploreDoc* CExploreTabbedView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CExploreDoc)));
	return (CExploreDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CExploreTabbedView message handlers

void CExploreTabbedView::OnContextMenu(CWnd*, CPoint point)
{
//	theApp.ShowPopupMenu(IDR_CONTEXT_MENU, point, this);
}

int CExploreTabbedView::AddView(CRuntimeClass* pViewClass, const CString& strViewLabel, int iIndex /*= -1*/, CCreateContext* pContext/* = NULL*/, BOOL bAfterCreation/* = FALSE*/)
{
	ASSERT_VALID(this);
	ENSURE(pViewClass != NULL);
	ENSURE(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CView)));

	CView* pView = DYNAMIC_DOWNCAST(CView, pViewClass->CreateObject());
	ASSERT_VALID(pView);

	if(! pView->Create(NULL, _T(""), WS_CHILD | WS_VISIBLE, CRect(0, 0, 0, 0), &m_wndTabs, (UINT)-1, pContext))
	{
//		TRACE1(_T("CTabView:Failed to create view '%s'\n"), pViewClass->m_lpszClassName);
		return -1;
	}

	CDocument* pDoc = GetDocument();
	if (pDoc != NULL)
	{
		ASSERT_VALID(pDoc);

		BOOL bFound = FALSE;
		for (POSITION pos = pDoc->GetFirstViewPosition();! bFound && pos != NULL;)
		{
			if(pDoc->GetNextView(pos) == pView)
				bFound = TRUE;
		}
		if(! bFound)
			pDoc->AddView(pView);
	}

	pView->SetParent(this);

	if(bAfterCreation)
		pView->SendMessage(WM_INITIALUPDATE, 0, 0);

	m_wndTabs.InsertTab(pView, strViewLabel, iIndex);

	int nTabs = m_wndTabs.GetTabsNum();
	return nTabs - 1;
}

int CExploreTabbedView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CTabView::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_wndTabs.DestroyWindow();
	if (! m_wndTabs.Create(CMFCTabCtrl::STYLE_3D, CRect(0, 0, 0, 0), this, 1, CMFCTabCtrl::LOCATION_BOTTOM))
	{
		TRACE(_T("Failed to create output tab window\n"));
		return -1;
	}

	m_wndTabs.SetFlatFrame();
	m_wndTabs.SetTabBorderSize(0);
	m_wndTabs.AutoDestroyWindow(FALSE);

	CMFCTabCtrl& MFCTabCtrl = GetTabControl();
	MFCTabCtrl.SetLocation((CMFCTabCtrl::Location)theApp.GetProfileInt(_T("Settings"), _T("ExploreTabTop"), CMFCBaseTabCtrl::LOCATION_BOTTOM));
	MFCTabCtrl.SetActiveTabBoldFont((BOOL)theApp.GetProfileInt(_T("Settings"), _T("ExploreTabBold"), 1));
	MFCTabCtrl.ModifyTabStyle(CMFCTabCtrl::STYLE_3D);
	MFCTabCtrl.EnableTabSwap(TRUE);
	MFCTabCtrl.EnableTabDocumentsMenu();
	MFCTabCtrl.HideActiveWindowHorzScrollBar();

	CMap<int, int, int, int> MapTabIndex;
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex1"), 0))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex1"), 0), TD_ENTITY_1);
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex2"), 1))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex2"), 1), TD_ENTITY_2);
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex3"), 2))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex3"), 2), TD_ENTITY_3);
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex4"), 3))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex4"), 3), TD_ENTITY_4);
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex5"), 4))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndex5"), 4), TD_ENTITY_5);
	if(100 != theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndexForm"), 5))
		MapTabIndex.SetAt(theApp.GetProfileInt(_T("Settings"), _T("ExploreTabIndexForm"), 5), TD_ENTITY_6);

	int nIndex = -1, nEntity = -1;
	for(int i = 0;i < 6;++i)
	{
		if(MapTabIndex.Lookup(i, nEntity))
			AddEntityView(nEntity);
	}

	return 0;
}

void CExploreTabbedView::OnDestroy()
{
	CMFCTabCtrl& MFCTabCtrl = GetTabControl();
	for(int i = 0;i < MFCTabCtrl.GetTabsNum();++i)
	{
		int r = i;
		CView* pView = (CView*)MFCTabCtrl.GetTabWnd(r);
		if(m_nIndexForm < 0 || (m_nIndexForm >= 0 && m_nIndexForm == r))
		{
			if(pView->IsKindOf(RUNTIME_CLASS(CTestFormView)))
				theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndexForm"), r);
		}
		if(m_nIndexForm >= 0 && r >= m_nIndexForm)
			r++;
		if(pView->IsKindOf(RUNTIME_CLASS(CExploreListView1)))
			theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndex1"), r);
		if(pView->IsKindOf(RUNTIME_CLASS(CExploreListView2)))
			theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndex2"), r);
		if(pView->IsKindOf(RUNTIME_CLASS(CExploreListView3)))
			theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndex3"), r);
		if(pView->IsKindOf(RUNTIME_CLASS(CExploreListView4)))
			theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndex4"), r);
		if(pView->IsKindOf(RUNTIME_CLASS(CExploreListView5)))
			theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndex5"), r);
	}
	if(m_nIndexForm > 0 && m_nIndexForm == MFCTabCtrl.GetTabsNum())
		theApp.WriteProfileInt(_T("Settings"), _T("ExploreTabIndexForm"), m_nIndexForm);

	CTabView::OnDestroy();

	// TODO: Add your message handler code here
}

BOOL CExploreTabbedView::OnEraseBkgnd(CDC* pDC) 
{
	return CTabView::OnEraseBkgnd(pDC);
	return TRUE;
}

void CExploreTabbedView::AddView1()
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CExploreListView1), _T("CListView1"), -1, &CContext);
}

void CExploreTabbedView::AddView2()
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CExploreListView2), _T("CListView2"), -1, &CContext);
}

void CExploreTabbedView::AddView3()
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CExploreListView3), _T("CListView3"), -1, &CContext);
}

void CExploreTabbedView::AddView4()
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CExploreListView4), _T("CListView4"), -1, &CContext);
}

void CExploreTabbedView::AddView5()
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CExploreListView5), _T("CListView5"), -1, &CContext);
}

void CExploreTabbedView::AddViewForm(int nIndex/* = -1*/)
{
	CCreateContext CContext;
	CContext.m_pCurrentFrame = NULL;
	CContext.m_pLastView = NULL;
	CContext.m_pNewDocTemplate = NULL;
	CContext.m_pCurrentDoc = GetDocument();
	AddView(RUNTIME_CLASS(CTestFormView), _T("CTestFormView"), nIndex, &CContext, nIndex > 0 ? TRUE : FALSE);
}

void CExploreTabbedView::AddEntityView(int nEntity)
{
	switch(nEntity)
	{
	case TD_ENTITY_1:
		AddView1();
		break;
	case TD_ENTITY_2:
		AddView2();
		break;
	case TD_ENTITY_3:
		AddView3();
		break;
	case TD_ENTITY_4:
		AddView4();
		break;
	case TD_ENTITY_5:
		AddView5();
		break;
	case TD_ENTITY_6:
		AddViewForm(m_nIndexForm);
		break;
	}
}

int CExploreTabbedView::RemoveViewForm()
{
	CMFCTabCtrl& MFCTabCtrl = GetTabControl();
	for(int i = 0;i < MFCTabCtrl.GetTabsNum();++i)
	{
		CView* pView = (CView*)MFCTabCtrl.GetTabWnd(i);
		if(pView->IsKindOf(RUNTIME_CLASS(CTestFormView)))
		{
			pView->ShowWindow(SW_HIDE);
			if(RemoveView(i))
				return i;
		}
	}

	return -1;
}

LRESULT CExploreTabbedView::OnHandleViewForm(WPARAM wParam, LPARAM lParam)
{
	int nTopicId = GetDocument()->GetTreeItemData();
	if(nTopicId < 1)
		return 0;

	if(2 == nTopicId)	// hide topic
	{
		if(m_nIndexForm < 0)	// has shown state
		{
			m_nIndexForm = RemoveViewForm();
		}
	}
	else						// show	topic
	{
		if(m_nIndexForm >= 0)
		{
			AddViewForm(m_nIndexForm);
			m_nIndexForm = -1;
		}
	}

	return 1;
}
