// SplitterWndExt.cpp : implementation file
//

#include "stdafx.h"
#include "SplitterWndExt.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplitterWndExt

CSplitterWndExt::CSplitterWndExt() 
	:m_nHiddenRow(-1)
	,m_nHiddenCol(-1)
{
}

CSplitterWndExt::~CSplitterWndExt()
{
}

BEGIN_MESSAGE_MAP(CSplitterWndExt, CSplitterWndEx)
	//{{AFX_MSG_MAP(CSplitterWndExt)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSplitterWndExt message handlers

void CSplitterWndExt::SetActivePane(int row, int col, CWnd* pWnd/* = NULL*/)
{
	// set the focus to the pane
	CWnd* pPane = pWnd == NULL ? GetPane(row, col) : pWnd;
	if(pPane->IsKindOf(RUNTIME_CLASS(CView)))
	{
		CFrameWnd* pFrameWnd = GetParentFrame();
		ASSERT_VALID(pFrameWnd);
		pFrameWnd->SetActiveView((CView*)pPane);
	}
	else if(pPane->IsKindOf(RUNTIME_CLASS(CSplitterWndEx))) // the pane is CSplitterWndEx window
	{
		CSplitterWndEx* pSplitter = (CSplitterWndEx*)pPane;
		CView* pView = (CView*)pSplitter->GetPane(0, 0);
		ASSERT_VALID(pView);
		CFrameWnd* pFrameWnd = GetParentFrame();
		ASSERT_VALID(pFrameWnd);
		pFrameWnd->SetActiveView(pView);
	}
	else
	{
		TRACE(_T("Warning: Next pane is not a view - calling SetFocus.\n"));
		pPane->SetFocus();
	}
}

void CSplitterWndExt::HideColumn(int nColToHide)
{
	ASSERT_VALID(this);
	ASSERT(m_nCols > 1);
	ASSERT(nColToHide < m_nCols);
	ASSERT(m_nHiddenCol == -1);
	m_nHiddenCol = nColToHide;
	// if the column has an active window -- change it
	int nActiveRow = 0, nActiveCol = 0;
	if(NULL != GetActivePane(&nActiveRow, &nActiveCol))
	{
		if(nActiveCol == nColToHide)
		{
			if(++nActiveCol >= m_nCols)
				nActiveCol = 0;
			SetActivePane(nActiveRow, nActiveCol);
		}
	}
	// hide all column panes
	for(int nRow = 0;nRow < m_nRows;++nRow)
	{
		CWnd* pPaneHide = GetPane(nRow, nColToHide);
		ASSERT(pPaneHide != NULL);
		pPaneHide->ShowWindow(SW_HIDE);
		pPaneHide->SetDlgCtrlID(AFX_IDW_PANE_FIRST + nRow * 16 + m_nCols);
		for(int nCol = nColToHide + 1;nCol < m_nCols;++nCol)
		{
			CWnd* pPane = GetPane(nRow, nCol);
			ASSERT(pPane != NULL);
			pPane->SetDlgCtrlID(IdFromRowCol(nRow, nCol - 1));
		}
	}

	m_nCols--;
	m_pColInfo[m_nCols].nCurSize = m_pColInfo[nColToHide].nCurSize;
	RecalcLayout();
}

void CSplitterWndExt::ShowHiddenColumn()
{
	ASSERT_VALID(this);
	ASSERT(m_nCols < m_nMaxCols);
	ASSERT(m_nHiddenCol != -1);

	int nShowCol = m_nHiddenCol;
	m_nHiddenCol = -1;

	int cxNew = m_pColInfo[m_nCols].nCurSize;
	m_nCols++;  // add a column

	ASSERT(m_nCols == m_nMaxCols);

	int nCol = 0;
	// Show the hidden column
	for(int nRow = 0;nRow < m_nRows;++nRow)
	{
		CWnd* pPaneShow = GetDlgItem(AFX_IDW_PANE_FIRST + nRow * 16 + m_nCols);
		ASSERT(pPaneShow != NULL);
		pPaneShow->ShowWindow(SW_SHOWNA);
		for(nCol = m_nCols - 2;nCol >= nShowCol;--nCol)
		{
			CWnd* pPane = GetPane(nRow, nCol);
			ASSERT(pPane != NULL);
			pPane->SetDlgCtrlID(IdFromRowCol(nRow, nCol + 1));
		}
		pPaneShow->SetDlgCtrlID(IdFromRowCol(nRow, nShowCol));
	}

	// new panes have been created -- recalculate layout
	for(nCol = nShowCol + 1;nCol < m_nCols;++nCol)
		m_pColInfo[nCol].nIdealSize = m_pColInfo[nCol - 1].nCurSize;

	m_pColInfo[nShowCol].nIdealSize = cxNew;
	RecalcLayout();
}

void CSplitterWndExt::HideRow(int nRowToHide)
{
	ASSERT_VALID(this);
	ASSERT(m_nRows > 1);
	ASSERT(nRowToHide < m_nRows);
	ASSERT(m_nHiddenRow == -1);
	m_nHiddenRow = nRowToHide;

	int nActiveRow = 0, nActiveCol = 0;
    // if the nRow has an active window -- change it
	if(GetActivePane(&nActiveRow, &nActiveCol) != NULL)
	{
		if(nActiveRow == nRowToHide)
		{
			if(++nActiveRow >= m_nRows)
				nActiveRow = 0;
			SetActivePane(nActiveRow, nActiveCol);
		}
	}
	// hide all nRow panes.
	for(int nCol = 0;nCol < m_nCols;++nCol)
	{
		CWnd* pPaneHide = GetPane(nRowToHide, nCol);
		ASSERT(pPaneHide != NULL);
		pPaneHide->ShowWindow(SW_HIDE);
		pPaneHide->SetDlgCtrlID(AFX_IDW_PANE_FIRST + nCol * 16 + m_nRows);
		for(int nRow = nRowToHide + 1;nRow < m_nRows;++nRow)
		{
			CWnd* pPane = GetPane(nRow, nCol);
			ASSERT(pPane != NULL);
			pPane->SetDlgCtrlID(IdFromRowCol(nRow - 1, nCol));
		}
	}

	m_nRows--;
	m_pRowInfo[m_nRows].nCurSize = m_pRowInfo[nRowToHide].nCurSize;
	RecalcLayout();
}

void CSplitterWndExt::ShowHiddenRow()
{
	ASSERT_VALID(this);
	ASSERT(m_nRows < m_nMaxRows);
	ASSERT(m_nHiddenRow != -1);

	int nShowRow = m_nHiddenRow;
	m_nHiddenRow = -1;

	int cyNew = m_pRowInfo[m_nRows].nCurSize;
	m_nRows++;  // add a nRow

	ASSERT(m_nRows == m_nMaxRows);

	int nRow = 0;
	// Show the hidden nRow
	for(int nCol = 0;nCol < m_nCols;++nCol)
	{
		CWnd* pPaneShow = GetDlgItem(AFX_IDW_PANE_FIRST + nCol * 16 + m_nRows);
		ASSERT(pPaneShow != NULL);
		pPaneShow->ShowWindow(SW_SHOWNA);
		for(nRow = m_nRows - 2;nRow >= nShowRow;--nRow)
		{
			CWnd* pPane = GetPane(nRow, nCol);
			ASSERT(pPane != NULL);
			pPane->SetDlgCtrlID(IdFromRowCol(nRow + 1, nCol));
		}
		pPaneShow->SetDlgCtrlID(IdFromRowCol(nShowRow, nCol));
	}
	// new panes have been created -- recalculate layout
	for(nRow = nShowRow + 1;nRow < m_nRows;++nRow)
		m_pRowInfo[nRow].nIdealSize = m_pRowInfo[nRow - 1].nCurSize;

	m_pRowInfo[nShowRow].nIdealSize = cyNew;
	RecalcLayout();
}

/////////////////////////////////////////////////////////////////////////////
// CSplitterWndExt message handlers
